/*
    MakeMKV GUI - Graphics user interface application for MakeMKV

    Copyright (C) 2009 GuinpinSoft inc <makemkvgui@makemkv.com>

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS"
    basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
    License for the specific language governing rights and limitations
    under the License.

*/
#include "aboutbox.h"
#include <QtGui/QtGui>
#include "qtapp.h"

CAboutBox::CAboutBox(QWidget *Parent,CApClient* App) : QDialog(Parent)
{
    m_app = App;

    setWindowTitle(UI_QSTRING(APP_ABOUTBOX_TITLE));

    // layout
    QGridLayout* lay = new QGridLayout();

    QString app_name = QStringFromUtf16(m_app->GetAppString(AP_vastr_Name));
    QString app_ver = QStringFromUtf16(m_app->GetAppString(AP_vastr_Version));
    QString app_arch = QStringFromUtf16(m_app->GetAppString(AP_vastr_Platform));
    QString app_keytype = QStringFromUtf16(m_app->GetAppString(AP_vastr_KeyType));
    QString app_keyfeatures = QStringFromUtf16(m_app->GetAppString(AP_vastr_KeyFeatures));
    QString app_keytime = QStringFromUtf16(m_app->GetAppString(AP_vastr_KeyExpiration));

    QString gui_verstr;
    bool    show_gui_version=true;
    
#if defined(APP_OFFICIAL_BUILD) || defined(HAVE_BUILDINFO_H)
#include <ver_num.h>

#if defined(APP_OFFICIAL_BUILD)
#include <archdefs.h>
    QString gui_ver = QString(QLatin1String(MAKEMKV_VERSION_NUMBER));
    QString gui_arch = QString(QLatin1String(ARCH_NAME));
#endif

#if defined(HAVE_BUILDINFO_H)
#include <gen_buildinfo.h>
    QString gui_ver = QString(QLatin1String(MAKEMKV_VERSION_NUMBER));
    QString gui_arch = QString(QLatin1String("for ")) + QString(QLatin1String(BUILDINFO_ARCH_NAME));
#endif

    if ( (gui_ver==app_ver) && (gui_arch==app_arch) )
    {
        show_gui_version=false;
    } else {
        gui_verstr = QString(QLatin1String("%1 [%2]"));
        gui_verstr=gui_verstr.arg(gui_ver,gui_arch);
    }
#else
    gui_verstr=QString(QLatin1String("custom build"));
#endif

    QString str_1 = QString(QLatin1String("%1 %2 [%3]"));
    str_1=str_1.arg(app_name,app_ver,app_arch);

    QString str_2 = UI_QSTRING(APP_IFACE_GUI_VERSION);
    str_2=str_2.arg(gui_verstr);

    lay->addWidget(createLabel(str_1),0,0,1,2,Qt::AlignCenter);
    if (show_gui_version)
    {
        lay->addWidget(createLabel(str_2),1,0,1,2,Qt::AlignCenter);
    }
    lay->addWidget(createHLine(),2,0,1,2);

    lay->addWidget(createLabel(UI_QSTRING(APP_IFACE_LICENSE_TYPE)),3,0,Qt::AlignRight);
    lay->addWidget(createLabel(app_keytype),3,1,Qt::AlignLeft);

    lay->addWidget(createLabel(UI_QSTRING(APP_IFACE_ENABLED_FEATURES)),4,0,Qt::AlignRight);
    lay->addWidget(createLabel(app_keyfeatures),4,1,Qt::AlignLeft);

    if (false==app_keytime.isEmpty())
    {
        lay->addWidget(createLabel(UI_QSTRING(APP_IFACE_EXPIRATION_DATE)),5,0,Qt::AlignRight);
        lay->addWidget(createLabel(app_keytime),5,1,Qt::AlignLeft);
    }

    lay->addWidget(createHLine(),6,0,1,2);

    QDialogButtonBox* btn_box = new QDialogButtonBox(QDialogButtonBox::Close,Qt::Horizontal);
    lay->addWidget(btn_box,7,1);

    setLayout(lay);

    connect(btn_box, SIGNAL(rejected()), this, SLOT(reject()));
}

