/*
    MakeMKV GUI - Graphics user interface application for MakeMKV

    Copyright (C) 2009 GuinpinSoft inc <makemkvgui@makemkv.com>

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS"
    basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
    License for the specific language governing rights and limitations
    under the License.

*/
#include <lgpl/aproxy.h>
#include <windows.h>
#include <winnt.h>
#include <lgpl/smem.h>

#define _LL1(x,y) x##y
#define _LL(x) _LL1(L,x)

extern bool AppEngMain(const char *ver);

static DWORD WINAPI DebugEngMainThreadProc(LPVOID lpParameter)
{
    AppEngMain((const char*)lpParameter);
    CloseHandle(GetStdHandle(STD_OUTPUT_HANDLE));
    return 0;
}

typedef BOOL (WINAPI *LPFN_ISWOW64PROCESS) (HANDLE, PBOOL);

static bool IsWow64()
{
    BOOL bIsWow64 = FALSE;
    LPFN_ISWOW64PROCESS fnIsWow64Process;

    if (0!=GetEnvironmentVariableA("APROXY_FORCE_32BIT",NULL,0))
    {
        return false;
    }

    fnIsWow64Process = (LPFN_ISWOW64PROCESS)GetProcAddress(
        GetModuleHandleW(L"KERNEL32"),"IsWow64Process");
  
    if (NULL != fnIsWow64Process)
    {
        if (!fnIsWow64Process(GetCurrentProcess(),&bIsWow64))
        {
            // handle error
            bIsWow64 = FALSE;
        }
    }
    return (FALSE!=bIsWow64);
}


char* ApSpawnApp()
{
    wchar_t appname[1024];
    wchar_t cmdline[64];

    if (GetModuleFileNameW(NULL,appname,1024)>1023) return NULL;
    
    wchar_t* app_end = appname + wcslen(appname) - 1;
    while( (*app_end!='\\') && (*app_end!='/') )
    {
        app_end--;
    }
    app_end++;

    if ( (IsWow64()) && false )
    {
        wcscpy(app_end,L"makemkvcon64.exe");
    } else {
        wcscpy(app_end,L"makemkvcon.exe");
    }

    memcpy(cmdline,L"xxx guiserver ",14*sizeof(wchar_t));
    wcscpy(cmdline+14,_LL(AP_VER));

    STARTUPINFOW si;
    PROCESS_INFORMATION pi;
    HANDLE  hin;
    SECURITY_ATTRIBUTES pipe_si;

    memset(&pipe_si,0,sizeof(pipe_si));
    pipe_si.nLength=sizeof(pipe_si);
    pipe_si.bInheritHandle=TRUE;

    memset(&si,0,sizeof(si));
    si.cb=sizeof(si);
    si.dwFlags = STARTF_USESTDHANDLES;

    if (0==CreatePipe(&hin,&si.hStdOutput,&pipe_si,0)) return false;

#ifdef APP_DEBUG_INPROC
    const char *av = AP_VER;
    SetStdHandle(STD_OUTPUT_HANDLE,si.hStdOutput);
    pi.hThread=CreateThread(NULL,0,DebugEngMainThreadProc,(LPVOID)av,0,NULL);
    if (NULL==pi.hThread) return false;
#else
    if (0==CreateProcessW(appname,cmdline,NULL,NULL,TRUE,DETACHED_PROCESS,NULL,NULL,&si,&pi)) return NULL;

    CloseHandle(pi.hThread);
    CloseHandle(pi.hProcess);
    CloseHandle(si.hStdOutput);
#endif

    // read answer
    static char reply[1024];
    for (unsigned int i=0;i<(sizeof(reply)-1);i++)
    {
        DWORD rd;

        do {
            if (0==ReadFile(hin,reply+i,1,&rd,NULL)) return NULL;
        } while(0==rd);

        if (reply[i]=='$')
        {
            reply[i]=0;

            // break pipe
            CloseHandle(hin);
            return reply;
        }
    }

    return NULL;
}

void* ApOpenShmem(const char *name)
{
    uintptr_t v=0;

    while(*name!=0)
    {
        v<<=4;
        v+= (*name-'a');
        name++;
    }

    HANDLE hs = (HANDLE)(void*)v;
    if (NULL==hs) return NULL;
    return MapViewOfFile(hs,FILE_MAP_WRITE,0,0,0);
}

bool ApOpenSgrp(AP_SGRP *sgrp,const uint64_t* data)
{
    sgrp->a_id = (uintptr_t) data[0];
    sgrp->b_id = (uintptr_t) data[1];

    return true;
}

bool ApSemInc(AP_SGRP *sgrp,uintptr_t sid)
{
    if (0==ReleaseSemaphore((HANDLE)(void*)sid,1,NULL)) return false;
    return true;
}

bool ApSemDec(AP_SGRP *sgrp,uintptr_t sid)
{
    DWORD r,w;

#ifdef APP_DEBUG_INPROC
    w = INFINITE;
#else
    w = 3000;
#endif

    r = WaitForSingleObject((HANDLE)(void*)sid,w);

    return (r==WAIT_OBJECT_0);
}

