/*
    MakeMKV GUI - Graphics user interface application for MakeMKV

    Copyright (C) 2009 GuinpinSoft inc <makemkvgui@makemkv.com>

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS"
    basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
    License for the specific language governing rights and limitations
    under the License.

*/
#include <QtGui/QtGui>
#include "settingdlg.h"
#include "mainwnd.h"
#include <lgpl/sstring.h>

CSettingDialog::CSettingDialog(CApClient* ap_client,QWidget *parent) : QDialog(parent)
{
    client = ap_client;

    show_restart_box = true;

    tabWidget = new QTabWidget();
    buttonBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel | QDialogButtonBox::Apply , Qt::Horizontal);

    // general
    generalTab = new CGeneralTab();
    tabWidget->addTab(generalTab , UI_QSTRING(APP_IFACE_SETTINGS_TAB_GENERAL));

    // IO
    ioTab = new CIOTab();
    tabWidget->addTab(ioTab, QLatin1String("IO"));

    // dvd
    dvdTab = new CDVDTab();
    tabWidget->addTab(dvdTab, QLatin1String("DVD"));

    // aacs
    aacsTab = new CAACSTab();
    tabWidget->addTab(aacsTab, QLatin1String("AACS"));

    // bd+
    bdpTab = new CBDPTab();
    tabWidget->addTab(bdpTab, QLatin1String("BD+"));

    QBoxLayout *lay = new QVBoxLayout();
    lay->addWidget(tabWidget);
    lay->addStretch(10);
    lay->addWidget(buttonBox);
    this->setLayout(lay);

    connect(buttonBox, SIGNAL(accepted()), this, SLOT(accept()));
    connect(buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
    connect(buttonBox->button(QDialogButtonBox::Apply), SIGNAL(clicked()), this, SLOT(SlotApply()));

    connect(this, SIGNAL(accepted()) , this , SLOT(SlotApply()));

    ReadSettings();
};

void CSettingDialog::SlotApply()
{
    if (false==WriteSettings())
    {
        QMessageBox::critical(this,UI_QSTRING(APP_CAPTION_MSG),UI_QSTRING(APP_IFACE_SETTINGS_MSG_FAILED));
    }
    ReadSettings();
    if (show_restart_box)
    {
        show_restart_box=false;
        QMessageBox::information(this,UI_QSTRING(APP_CAPTION_MSG),
            UI_QSTRING(APP_IFACE_SETTINGS_MSG_RESTART));
    }
}

void CSettingDialog::ReadSettings()
{
    static const utf16_t zero[1]={0};

    // general
    const utf16_t *dest_path = client->GetSettingString(apset_app_DestinationDir);
    if (NULL==dest_path) dest_path=zero;
    generalTab->destinationDir->setText(QStringFromUtf16(dest_path));

    int dest_type = client->GetSettingInt(apset_app_DestinationType);
    generalTab->destinationDir->setIndexValue(dest_type);

    int show_debug = client->GetSettingInt(apset_app_ShowDebug);
    generalTab->check_DebugLog->setCheckState( (show_debug==0) ? Qt::Unchecked : Qt::Checked );

    // io
    int retry_count = client->GetSettingInt(apset_io_ErrorRetryCount);
    ioTab->spinBoxRetryCount->setValue(retry_count);
    int buf_size = client->GetSettingInt(apset_io_RBufSizeMB);

    if (0==buf_size)
    {
        ioTab->comboBoxRBufSize->setCurrentIndex(0);
    } else {
        char sbuf[32];
        sprintf_s(sbuf,32,"%u",buf_size);
        QString sstr=QLatin1String(sbuf);
        if (ioTab->comboBoxRBufSize->findText(sstr)<0)
        {
            ioTab->comboBoxRBufSize->addItem(sstr);
        }
        ioTab->comboBoxRBufSize->setCurrentIndex(ioTab->comboBoxRBufSize->findText(sstr));
    }

    // dvd
    int MinimumTitleLength = client->GetSettingInt(apset_dvd_MinimumTitleLength);
    dvdTab->spinBoxMinimumTitleLength->setValue(MinimumTitleLength);

    int SpRemoveMethod = client->GetSettingInt(apset_dvd_SPRemoveMethod);
    dvdTab->comboBoxSpRemoveMethod->setCurrentIndex(SpRemoveMethod);

    // aacs
    const utf16_t *keypath = client->GetSettingString(apset_aacs_KeyFilesDir);
    if (NULL==keypath) keypath=zero;
    aacsTab->keyDir->setText(QStringFromUtf16(keypath));

    // bd+
    const utf16_t *svqpath = client->GetSettingString(apset_bdplus_SvqDir);
    if (NULL==svqpath) svqpath=zero;
    bdpTab->svqDir->setText(QStringFromUtf16(svqpath));

    const utf16_t *dumppath = client->GetSettingString(apset_bdplus_DumpDir);
    if (NULL==dumppath) dumppath=zero;
    bdpTab->dumpDir->setText(QStringFromUtf16(dumppath));
}

bool CSettingDialog::WriteSettings()
{
    // general
    client->SetSettingString(apset_app_DestinationDir,Utf16FromQString(generalTab->destinationDir->text()));
    client->SetSettingInt(apset_app_DestinationType,generalTab->destinationDir->getIndexValue());

    client->SetSettingInt( apset_app_ShowDebug , (generalTab->check_DebugLog->checkState() == Qt::Checked) ? 1 : 0 );

    // io
    client->SetSettingInt(apset_io_ErrorRetryCount,ioTab->spinBoxRetryCount->value());
    int rbuf_size;
    if (ioTab->comboBoxRBufSize->currentIndex()==0)
    {
        rbuf_size=0;
    } else {
        rbuf_size=ioTab->comboBoxRBufSize->currentText().toInt();
    }
    client->SetSettingInt(apset_io_RBufSizeMB,rbuf_size);

    // dvd
    client->SetSettingInt(apset_dvd_MinimumTitleLength,dvdTab->spinBoxMinimumTitleLength->value());
    client->SetSettingInt(apset_dvd_SPRemoveMethod,dvdTab->comboBoxSpRemoveMethod->currentIndex());

    // aacs
    client->SetSettingString(apset_aacs_KeyFilesDir,Utf16FromQString(aacsTab->keyDir->text()));

    // bd+
    client->SetSettingString(apset_bdplus_SvqDir,Utf16FromQString(bdpTab->svqDir->text()));
    client->SetSettingString(apset_bdplus_DumpDir,Utf16FromQString(bdpTab->dumpDir->text()));

    // flush
    return client->SaveSettings();
}

CIOTab::CIOTab(QWidget *parent) : QWidget(parent)
{
    QGroupBox* box = new QGroupBox(UI_QSTRING(APP_IFACE_SETTINGS_IO_OPTIONS));

    spinBoxRetryCount = new QSpinBox();
    comboBoxRBufSize = new QComboBox();
    comboBoxRBufSize->addItem(QString(UI_QSTRING(APP_IFACE_SETTINGS_IO_AUTO)));
    comboBoxRBufSize->addItem(QLatin1String("64"));
    comboBoxRBufSize->addItem(QLatin1String("256"));
    comboBoxRBufSize->addItem(QLatin1String("512"));
    comboBoxRBufSize->addItem(QLatin1String("1024"));
    comboBoxRBufSize->addItem(QLatin1String("2048"));

    QGridLayout *b_lay = new QGridLayout();
    b_lay->setColumnStretch(0,2);

    b_lay->addWidget(createLabel(UI_QSTRING(APP_IFACE_SETTINGS_IO_READ_RETRY)),0,0,Qt::AlignRight);
    b_lay->addWidget(spinBoxRetryCount,0,1);
    b_lay->addWidget(createLabel(UI_QSTRING(APP_IFACE_SETTINGS_IO_READ_BUFFER)),1,0,Qt::AlignRight);
    b_lay->addWidget(comboBoxRBufSize,1,1);
    box->setLayout(b_lay);

    QBoxLayout *lay = new QVBoxLayout();
    lay->addWidget(box);

    lay->addStretch(2);
    this->setLayout(lay);
}

CDVDTab::CDVDTab(QWidget *parent) : QWidget(parent)
{
    QGroupBox* box = new QGroupBox(UI_QSTRING(APP_IFACE_SETTINGS_DVD_OPTIONS));

    spinBoxMinimumTitleLength = new QSpinBox();
    spinBoxMinimumTitleLength->setMaximum(9999);
    comboBoxSpRemoveMethod = new QComboBox();
    comboBoxSpRemoveMethod->addItem(UI_QSTRING(APP_IFACE_SETTINGS_DVD_AUTO));
    comboBoxSpRemoveMethod->addItem(QLatin1String("CellWalk"));
    comboBoxSpRemoveMethod->addItem(QLatin1String("CellTrim"));

    QGridLayout *b_lay = new QGridLayout();
    b_lay->setColumnStretch(0,2);

    b_lay->addWidget(createLabel(UI_QSTRING(APP_IFACE_SETTINGS_DVD_MIN_LENGTH)),0,0,Qt::AlignRight);
    b_lay->addWidget(spinBoxMinimumTitleLength,0,1);

    b_lay->addWidget(createLabel(UI_QSTRING(APP_IFACE_SETTINGS_DVD_SP_REMOVE)),1,0,Qt::AlignRight);
    b_lay->addWidget(comboBoxSpRemoveMethod,1,1);

    box->setLayout(b_lay);
    QBoxLayout *lay = new QVBoxLayout();
    lay->addWidget(box);
    lay->addStretch(2);
    this->setLayout(lay);
}

CAACSTab::CAACSTab(QWidget *parent) : QWidget(parent)
{
    keyDir = new CDirSelectBox(UI_QSTRING(APP_IFACE_SETTINGS_AACS_KEY_DIR));

    QBoxLayout *lay = new QVBoxLayout();

    lay->addWidget(keyDir);

    lay->addStretch(2);
    this->setLayout(lay);
}

CBDPTab::CBDPTab(QWidget *parent) : QWidget(parent)
{
    svqDir = new CDirSelectBox(UI_QSTRING(APP_IFACE_SETTINGS_BDP_SVQ_DIR));
    dumpDir = new CDirSelectBox(UI_QSTRING(APP_IFACE_SETTINGS_BDP_DUMP_DIR));

    QBoxLayout *lay = new QVBoxLayout();

    lay->addWidget(svqDir);
    lay->addWidget(dumpDir);

    lay->addStretch(2);
    this->setLayout(lay);
}


CGeneralTab::CGeneralTab(QWidget *parent) : QWidget(parent)
{
    radio_None = new QRadioButton(UI_QSTRING(APP_IFACE_SETTINGS_DEST_TYPE_NONE));
    radio_Auto = new QRadioButton(UI_QSTRING(APP_IFACE_SETTINGS_DEST_TYPE_AUTO));
    radio_SemiAuto = new QRadioButton(UI_QSTRING(APP_IFACE_SETTINGS_DEST_TYPE_SEMIAUTO));
    radio_Custom = new QRadioButton(UI_QSTRING(APP_IFACE_SETTINGS_DEST_TYPE_CUSTOM));

    QWidgetList lst;
    lst.append(radio_None);
    lst.append(radio_Auto);
    lst.append(radio_SemiAuto);
    lst.append(radio_Custom);

    destinationDir = new CDirSelectBox(UI_QSTRING(APP_IFACE_SETTINGS_DESTDIR),lst);

    connect(destinationDir, SIGNAL(SignalIndexChanged()) , this , SLOT(SlotIndexChanged()) );

    QGroupBox* miscBox = new QGroupBox(UI_QSTRING(APP_IFACE_SETTINGS_GENERAL_MISC));
    QGridLayout *m_lay = new QGridLayout();
    m_lay->setColumnStretch(0,2);

    check_DebugLog = new QCheckBox();
    m_lay->addWidget(createLabel(UI_QSTRING(APP_IFACE_SETTINGS_LOG_DEBUG_MSG)),0,0,Qt::AlignRight);
    m_lay->addWidget(check_DebugLog,0,1);
    miscBox->setLayout(m_lay);

    QBoxLayout *lay = new QVBoxLayout();
    lay->addWidget(destinationDir);
    lay->addWidget(miscBox);

    lay->addStretch(2);

    this->setLayout(lay);
}

void CGeneralTab::SlotIndexChanged()
{
    int index = destinationDir->getIndexValue();
    if ( (index==0) || (index==1) )
    {
        destinationDir->setDirEnabled(false);
    }
    if ( (index==2) || (index==3) )
    {
        destinationDir->setDirEnabled(true);
    }
}




